@@ -118,6 +118,47 @@ macro_rules! impl_item_arg3 {
     };
 }
 
+#[macro_export]
+macro_rules! impl_item_arg4 {
+    ($input:expr, $name:expr, $arguments:expr, $item_type:ty) => {
+        if $name == stringify!($item_type) {
+            use nom::Err as NomErr;
+
+            use crate::error::{ErrorKind, Expectation, ParserError};
+
+            return if $arguments.len() == 4 {
+                Ok((
+                    $input,
+                    Box::new(<$item_type>::new(
+                        $arguments[0]
+                            .clone()
+                            .try_into()
+                            .map_err(|err| Err::Error(err))?,
+                        $arguments[1]
+                            .clone()
+                            .try_into()
+                            .map_err(|err| Err::Error(err))?,
+                        $arguments[2]
+                            .clone()
+                            .try_into()
+                            .map_err(|err| Err::Error(err))?,
+                        $arguments[3]
+                            .clone()
+                            .try_into()
+                            .map_err(|err| Err::Error(err))?,
+                    )),
+                ))
+            } else {
+                Err(NomErr::Error(ParserError::Base {
+                    location: $input,
+                    kind: ErrorKind::Expected(Expectation::ArgumentCount(4, $arguments.len())),
+                    child: None,
+                }))
+            };
+        }
+    };
+}
+
 #[macro_export]
 macro_rules! impl_tests_for_item_arg0 {
     ($test_module_name:ident, $item:ident, $item_type:ty) => {
@@ -178,7 +219,7 @@ macro_rules! impl_tests_for_item_arg0 {
 
 #[macro_export]
 macro_rules! impl_tests_for_item_arg1 {
-    ($test_module_name:ident, $item:ident, $item_type:ty, ($argument0:expr, $argument0_value:expr)) => {
+    ($test_module_name:ident, $item:ident, $item_type:ty, ($argument_or_constant0:expr, $argument_or_constant0_value:expr)) => {
         mod $test_module_name {
             use super::*;
 
@@ -193,7 +234,7 @@ macro_rules! impl_tests_for_item_arg1 {
                 let (input, item) = $item(&constants)(concat!(
                     stringify!($item_type),
                     "(",
-                    $argument0,
+                    $argument_or_constant0,
                     ");input"
                 ))
                 .unwrap();
@@ -201,7 +242,7 @@ macro_rules! impl_tests_for_item_arg1 {
                 assert_eq!(input, "input");
                 assert_eq!(
                     format!("{:?}", item),
-                    format!("{:?}", <$item_type>::new($argument0_value))
+                    format!("{:?}", <$item_type>::new($argument_or_constant0_value))
                 );
             }
 
@@ -209,7 +250,7 @@ macro_rules! impl_tests_for_item_arg1 {
             fn missing_semicolon_test() {
                 let constants = BTreeMap::new();
                 assert_matches!(
-                    $item(&constants)(concat!(stringify!($item_type), "(", $argument0, ")input")),
+                    $item(&constants)(concat!(stringify!($item_type), "(", $argument_or_constant0, ")input")),
                     Err(NomErr::Error(ParserError::Base {
                         location,
                         kind,
@@ -263,7 +304,7 @@ macro_rules! impl_tests_for_item_arg1 {
 
 #[macro_export]
 macro_rules! impl_tests_for_item_arg2 {
-    ($test_module_name:ident, $item:ident, $item_type:ty, ($argument0:expr, $argument0_value:expr), ($argument1:expr, $argument1_value:expr)) => {
+    ($test_module_name:ident, $item:ident, $item_type:ty, ($argument_or_constant0:expr, $argument_or_constant0_value:expr), ($argument1:expr, $argument1_value:expr)) => {
         mod $test_module_name {
             use super::*;
 
@@ -278,7 +319,7 @@ macro_rules! impl_tests_for_item_arg2 {
                 let (input, item) = $item(&constants)(concat!(
                     stringify!($item_type),
                     "(",
-                    $argument0,
+                    $argument_or_constant0,
                     ", ",
                     $argument1,
                     ");input"
@@ -290,7 +331,7 @@ macro_rules! impl_tests_for_item_arg2 {
                     format!("{:?}", item),
                     format!(
                         "{:?}",
-                        <$item_type>::new($argument0_value, $argument1_value)
+                        <$item_type>::new($argument_or_constant0_value, $argument1_value)
                     )
                 );
             }
@@ -302,7 +343,7 @@ macro_rules! impl_tests_for_item_arg2 {
                     $item(&constants)(concat!(
                         stringify!($item_type),
                         "( ",
-                        $argument0,
+                        $argument_or_constant0,
                         " , ",
                         $argument1,
                         " )input"
@@ -360,7 +401,7 @@ macro_rules! impl_tests_for_item_arg2 {
 
 #[macro_export]
 macro_rules! impl_tests_for_item_arg3 {
-    ($test_module_name:ident, $item:ident, $item_type:ty, ($argument0:expr, $argument0_value:expr), ($argument1:expr, $argument1_value:expr), ($argument2:expr, $argument2_value:expr)) => {
+    ($test_module_name:ident, $item:ident, $item_type:ty, ($argument_or_constant0:expr, $argument_or_constant0_value:expr), ($argument1:expr, $argument1_value:expr), ($argument2:expr, $argument2_value:expr)) => {
         mod $test_module_name {
             use super::*;
 
@@ -375,7 +416,7 @@ macro_rules! impl_tests_for_item_arg3 {
                 let (input, item) = $item(&constants)(concat!(
                     stringify!($item_type),
                     "( ",
-                    $argument0,
+                    $argument_or_constant0,
                     " , ",
                     $argument1,
                     " , ",
@@ -389,7 +430,7 @@ macro_rules! impl_tests_for_item_arg3 {
                     format!("{:?}", item),
                     format!(
                         "{:?}",
-                        <$item_type>::new($argument0_value, $argument1_value, $argument2_value)
+                        <$item_type>::new($argument_or_constant0_value, $argument1_value, $argument2_value)
                     )
                 );
             }
@@ -401,7 +442,7 @@ macro_rules! impl_tests_for_item_arg3 {
                     $item(&constants)(concat!(
                         stringify!($item_type),
                         "( ",
-                        $argument0,
+                        $argument_or_constant0,
                         " , ",
                         $argument1,
                         " , ",
@@ -458,3 +499,108 @@ macro_rules! impl_tests_for_item_arg3 {
         }
     };
 }
+
+#[macro_export]
+macro_rules! impl_tests_for_item_arg4 {
+    ($test_module_name:ident, $item:ident, $item_type:ty, ($argument_or_constant0:expr, $argument_or_constant0_value:expr), ($argument1:expr, $argument1_value:expr), ($argument2:expr, $argument2_value:expr), ($argument3:expr, $argument3_value:expr)) => {
+        mod $test_module_name {
+            use super::*;
+
+            use cool_asserts::assert_matches;
+            use nom::Err as NomErr;
+
+            use crate::error::{ParserError, Expectation, ErrorKind};
+
+            #[test]
+            fn test() {
+                let constants = BTreeMap::new();
+                let (input, item) = $item(&constants)(concat!(
+                    stringify!($item_type),
+                    "( ",
+                    $argument_or_constant0,
+                    " , ",
+                    $argument1,
+                    " , ",
+                    $argument2,
+                    " , ",
+                    $argument3,
+                    " );input"
+                ))
+                .unwrap();
+
+                assert_eq!(input, "input");
+                assert_eq!(
+                    format!("{:?}", item),
+                    format!(
+                        "{:?}",
+                        <$item_type>::new($argument_or_constant0_value, $argument1_value, $argument2_value, $argument3_value)
+                    )
+                );
+            }
+
+            #[test]
+            fn missing_semicolon_test() {
+                let constants = BTreeMap::new();
+                assert_matches!(
+                    $item(&constants)(concat!(
+                        stringify!($item_type),
+                        "( ",
+                        $argument_or_constant0,
+                        " , ",
+                        $argument1,
+                        " , ",
+                        $argument2,
+                        " , ",
+                        $argument3,
+                        " )input"
+                    )),
+                    Err(NomErr::Error(ParserError::Base {
+                        location,
+                        kind,
+                        child,
+                    })) => {
+                        assert_matches!(location, "input");
+                        assert_matches!(kind, ErrorKind::Expected(Expectation::Symbol(';')));
+                        assert_matches!(child, None);
+                    }
+                );
+            }
+
+            #[test]
+
+            fn too_few_arguments_test() {
+                let constants = BTreeMap::new();
+                assert_matches!(
+                    $item(&constants)(concat!(stringify!($item_type), "(false, false, false);input")),
+                    Err(NomErr::Error(ParserError::Base {
+                        location,
+                        kind,
+                        child,
+                    })) => {
+                        assert_matches!(location, "input");
+                        assert_matches!(kind, ErrorKind::Expected(Expectation::ArgumentCount(4, 3)));
+                        assert_matches!(child, None);
+                    }
+                );
+            }
+
+            #[test]
+
+            fn too_many_arguments_test() {
+                let constants = BTreeMap::new();
+                assert_matches!(
+                    $item(&constants)(concat!(stringify!($item_type), "(false, false, false, false, false);input")),
+                    Err(NomErr::Error(ParserError::Base {
+                        location,
+                        kind,
+                        child,
+                    })) => {
+                        assert_matches!(location, "input");
+                        assert_matches!(kind, ErrorKind::Expected(Expectation::ArgumentCount(4, 5)));
+                        assert_matches!(child, None);
+                    }
+                );
+            }
+        }
+    };
+}
